import hashlib
from django.test import TestCase
from guardian.shortcuts import assign_perm
from guardian.utils import get_anonymous_user

from pgp import models
from pgp.zbase32 import encode

class CorsHeadersTestCase(TestCase):
    """Check that responses have appropriate CORS headers"""

    def setUp(self):
        self.toykey = models.PublicKey.objects.create(
            email="frank@sauerburger.com"
        )
        anonymous = get_anonymous_user()
        assign_perm('view_publickey', anonymous, self.toykey)

    def test_policy(self):
        """Check that the policy file has CORS headers"""
        response = self.client.get('/.well-known/openpgpkey/policy')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response["access-control-allow-origin"], "*")

    def test_policy_advanced(self):
        """Check that the policy file has CORS headers"""
        response = self.client.get('/.well-known/openpgpkey/sauerburger.com/policy')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response["access-control-allow-origin"], "*")

    def test_mail_direct(self):
        """Check that a key file has CORS headers"""
        response = self.client.get('/.well-known/openpgpkey/'
                                   'hu/o4wcfswfr6ohpfhm51upxgdxhffqnt96',
                                   HTTP_HOST="sauerburger.com")
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response["access-control-allow-origin"], "*")

    def test_mail_advanced(self):
        """Check that a key file has CORS headers"""
        response = self.client.get("/.well-known/openpgpkey/sauerburger.com/"
                                   "hu/o4wcfswfr6ohpfhm51upxgdxhffqnt96")
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response["access-control-allow-origin"], "*")

class WkdCapitalizationTestCase(TestCase):
    """Check that wkd request capitalization is handled correctly"""
    def setUp(self):
        """Create toykey without pgp data"""
        self.toykey = models.PublicKey.objects.create(
            email="FrÄnk@SauerburgEr.com"
        )
        anonymous = get_anonymous_user()
        assign_perm('view_publickey', anonymous, self.toykey)

    def test_advanced(self):
        """Check that advanced url"""
        digest = hashlib.sha1("frÄnk".encode('utf-8')).digest()
        local_hash = encode(digest).decode()
        response = self.client.get("/.well-known/openpgpkey/sauerburger.com/"
                                   f"hu/{local_hash}")
        self.assertEqual(response.status_code, 200)

    def test_direct(self):
        """Check that advanced url"""
        digest = hashlib.sha1("frÄnk".encode('utf-8')).digest()
        local_hash = encode(digest).decode()
        response = self.client.get("/.well-known/openpgpkey/"
                                   f"hu/{local_hash}",
                                   HTTP_HOST="sauerburger.com")
        self.assertEqual(response.status_code, 200)
